(define input-files
  '("all.scm"
    "utilities.scm"
    "sort.scm"
    "linear-algebra.scm"
    "sparse-linear-algebra.scm"
    "points.scm"
    "geometry.scm"
    "geometry-code.scm"
    "linear-elements.scm"
    "linear-elements-code.scm"
    "problem-descriptions.scm"
    "conjugate-gradient.scm"
    "intergrid.scm"
    "multigrid.scm"
    "dirichlet.scm"
    "parabolic.scm"
    "mmoc.scm"
;;    "obstacle.scm"
    "polygon.scm"
    "pgm.scm"
    "final-tests.scm"
    "test-dirichlet.scm"
;;    "final-tests-obstacle.scm"
;;    "test.scm"
    "test-parabolic.scm"
    "test-mmoc.scm"
    "symbolic.scm"
 ;;   "test-matrix-and-rhs.scm"
    "tex.scm"
    "html-lib.scm"
    "describe.scm"))

(include "html-lib.scm")

(define (display-objects title os)
  (if (null? os)
      os
      (<li>
       title ":"
       (<ul>
	(map (lambda (o)
	       (let ((port (open-output-string)))
		 (define (my-display o)
		   (cond ((null? o) (display "()" port))
			 ((pair? o)
			  (display "(" port)
			  (my-display (car o))
			  (let loop ((rest (cdr o)))
			    (cond ((pair? rest)
				   (display " " port)
				   (my-display (car rest))
				   (loop (cdr rest)))
				  ((null? rest)
				   (display ")" port))
				  (else
				   (display " . " port)
				   (my-display rest)
				   (display ")" port)))))
			 (else
			  (display o port))))
		 (my-display o)
		 (<li> (<tt> (get-output-string port)))))
	     os)))))

(define (all-null? l)
  (or (null? l)
      (and (null? (car l))
	   (all-null? (cdr l)))))

(define (category-name c)
  (vector-ref c 0))

(define (category-recognizer c)
  (vector-ref c 1))

(define (category-extractor c)
  (vector-ref c 2))

(define categories
  ;; assumes that the form is a pair.
  ;; contains first the name, then the recognizer, then the extractor
  (list (vector "Classes"
		(lambda (form) (eq? (car form) 'define-class))
		(lambda (form) (cadr form)))     ; class name
	(vector "Generic functions"
		(lambda (form) (eq? (car form) 'define-generic))
		(lambda (form) (cadr form)))     ; name and argument list
	(vector "Methods"
		(lambda (form) (or (eq? (car form) 'define-method)
				   (eq? (car form) 'define-handy-method)))
		(lambda (form) (cadr form)))     ; name and argument list
	(vector "Functions"
		(lambda (form) (and (eq? (car form) 'define)
				    (pair? (cdr form))
				    (pair? (cadr form))))
		(lambda (form) (cadr form)))     ; name and argument list
	(vector "Other objects"
		(lambda (form) (and (eq? (car form) 'define)
				    (pair? (cdr form))
				    (symbol? (cadr form))))
		(lambda (form) (cadr form)))
	(vector "Pervasive macros"
		(lambda (form) (eq? (car form) 'define-meroon-macro))
		(lambda (form) (cadr form)))     ; name and argument list
	(vector "Local macros"
		(lambda (form) (or (eq? (car form) 'define-internal-meroon-macro)
				   (eq? (car form) 'define-macro)))
		(lambda (form) (cadr form)))
	(vector "Includes"
		(lambda (form) (eq? (car form) 'include))
		(lambda (form) (cadr form)))
	(vector "HTML tags"   ; name
		(lambda (form) (eq? (car form) 'define-tag))
		(lambda (form) (cadr form)))))   ; name



(define (get-forms)
  (let loop ((files input-files) (results '()))
    (if (null? files)
	(reverse results)
	(let ((port (open-input-file (car files))))
	  (let inner ((forms '()) (form (read port)))
	    (if (eof-object? form)
		(begin
		  (close-input-port port)
		  (loop (cdr files) (cons (cons (car files) (reverse forms)) results)))
		(inner (cons form forms)
		       (read port))))))))

(define *base* "http://www.math.purdue.edu/~lucier/615-2014/")

(define (base-dir .  url)
  (list *base* url))

(define (software-dir .  url)
  url)

(define (make-page all-forms)
(list
  (<unprotected> "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">")
  (<html>
   (<head> 
    (<title> 
     "Software for Numerical Methods for Partial Differential Equations"))
   (<body>
    (<center> 
     (<b> 
      (<font> size: "+3" 
	      "Software for Numerical Methods for Partial Differential Equations")))
    (<p> 
     "
This software was developed for and by the students in "(<a> href: (base-dir) "CS 615")", Numerical Methods for
Partial Differential Equations in 2000, 2003, 2005, 2008, 2010, and now 2012.  Its goal is to implement the
finite element method in two dimensions.  It is written in Scheme, and uses some features of 
"(<a> href: "http://dynamo.iro.umontreal.ca/~gambit/wiki/index.php/Main_Page" "Gambit")",
the Scheme->C compiler written by Marc Feeley.  It uses "(<a> href: (software-dir "MeroonV3-2012Jan20.tar.bz2")"Meroon")", developed by Christian Queinnec, as its
object system.")
    (<p>
     (<b> "Note: ")
     "
This software is a work in progress.  While much of it works as intended, much of it has not yet even
been tested.")
    
    
    
    (<p>
     "The code is coming along nicely and passes the tests in "
     (<tt> "final-tests.scm") " and " (<tt> "test-parabolic.scm")
     ". 
Note that with the infrastructure that we have built, it is trivial to compute
things like the L2 and H1 norms of the error.")
    (<p>
     "
Timing and accuracy results  are available for the multigrid code applied
to problems of varying difficulty in the file "
     (<a> href: (software-dir "mg-results")  (<b> "mg-results."))
     "
Some results on bigger problems can be found in "
     (<a> href: (software-dir "mg-results2")  (<b> "mg-results2.")))
    (<p>
     "
Timing and accuracy results are available for the conjugate-gradient code applied
to problems of varying difficulty in the file "
     (<a> href: (software-dir "cg-results")  (<b> "cg-results.")))
    (<p>  "The file " (<a> href: (software-dir "parabolic.scm") "parabolic.scm")
	  " contains a solver for parabolic differential equations,
and the file " (<a> href: (software-dir "mmoc.scm") "mmoc.scm") " contains a solver for
transport-dominated diffusion problems.  The last two files on this page were used to produce this HTML file.")
    (<p>  "A gzipped tarball of all the sources can be found "
	  (<a> href: (software-dir "PUFS-2014-05-01.tar.gz") "here")". A short article describing the process of software
development can be found "(<a> href: (base-dir "pde-in-scheme.pdf") "here")".")
    (<p> (<a> href: (software-dir "Changelog")  (<b> "Changelog")))
    (map (lambda (file-name+forms)
	   (let ((file-name (car file-name+forms))
		 (forms (cdr file-name+forms)))
	     (let ((contents (map (lambda (c) '()) categories)))
	       (let loop ((forms forms))
		 (if (not (null? forms))
		     (let ((form (car forms)))
		       (if (pair? form)
			   (let inner ((categories categories)
				       (contents contents))
			     (if (null? categories)
				 (loop (cdr forms))
				 (let ((c (car categories)))
				   (if ((category-recognizer c) form)
				       (begin
					 (set-car! contents (cons ((category-extractor c) form)
								  (car contents)))
					 (loop (cdr forms)))
				       (inner (cdr categories) 
					      (cdr contents))))))
			   (loop (cdr forms))))
		     (list (<p>
			    (<a> href: (software-dir file-name) (<b> file-name)))
			   (if (all-null? contents)
			       '()
			       (<ul>
				(map (lambda (category contents)
				       (display-objects (category-name category)
							(reverse contents)))
				     categories contents)))))))))
	 all-forms)))))
  
  (define (display-page page)
    (with-output-to-file "index.html"
      (lambda ()
        (html-display page))))

(define (build-index)
  (let* ((all-forms (time (get-forms)))
         (page (time (make-page all-forms))))
    (time (display-page page))))

(build-index)
